@@ -14,6 +14,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import annotations
+
 import collections
 import math
 import mmap
@@ -21,11 +23,12 @@ import os
 import re
 import struct
 import threading
+import typing
 
 import chess
 
 from types import TracebackType
-from typing import Deque, Dict, Iterable, Iterator, List, MutableMapping, Optional, Tuple, Type, TypeVar, Union
+from typing import Deque, Dict, Iterable, Iterator, List, Optional, Tuple, Type, TypeVar, Union
 
 
 UINT64_BE = struct.Struct(">Q")
@@ -370,16 +373,13 @@ PCHR = ["K", "Q", "R", "B", "N", "P"]
 TABLENAME_REGEX = re.compile(r"^[KQRBNP]+v[KQRBNP]+\Z")
 
 
-def is_tablename(name: str, *, one_king: bool = True, piece_count: Optional[int] = 7, normalized: bool = True) -> bool:
+def is_tablename(name: str, *, one_king: bool = True, piece_count: Optional[int] = TBPIECES, normalized: bool = True) -> bool:
     return (
         (piece_count is None or len(name) <= piece_count + 1) and
         bool(TABLENAME_REGEX.match(name)) and
         (not normalized or normalize_tablename(name) == name) and
         (not one_king or (name != "KvK" and name.startswith("K") and "vK" in name)))
 
-def is_table_name(name: str) -> bool:  # TODO: Remove before 1.0
-    return is_tablename(name, one_king=False)
-
 
 def tablenames(*, one_king: bool = True, piece_count: int = 6) -> Iterator[str]:
     first = "K" if one_king else "P"
@@ -622,11 +622,15 @@ class Table:
                     pass
 
     def check_magic(self, magic: Optional[bytes], pawnless_magic: Optional[bytes]) -> None:
+        assert self.data
+
         valid_magics = [magic, self.has_pawns and pawnless_magic]
         if self.data[:min(4, len(self.data))] not in valid_magics:
             raise IOError(f"invalid magic header: ensure {self.path!r} is a valid syzygy tablebase file")
 
     def setup_pairs(self, data_ptr: int, tb_size: int, size_idx: int, wdl: int) -> PairsData:
+        assert self.data
+
         d = PairsData()
 
         self._flags = self.data[data_ptr]
@@ -681,7 +685,7 @@ class Table:
 
         return d
 
-    def set_norm_piece(self, norm: List[int], pieces) -> None:
+    def set_norm_piece(self, norm: List[int], pieces: List[int]) -> None:
         if self.enc_type == 0:
             norm[0] = 3
         elif self.enc_type == 2:
@@ -748,7 +752,7 @@ class Table:
 
         return fac
 
-    def set_norm_pawn(self, norm: List[int], pieces) -> None:
+    def set_norm_pawn(self, norm: List[int], pieces: List[int]) -> None:
         norm[0] = self.pawns[0]
         if self.pawns[1]:
             norm[self.pawns[0]] = self.pawns[1]
@@ -762,6 +766,8 @@ class Table:
             i += norm[i]
 
     def calc_symlen(self, d: PairsData, s: int, tmp: List[int]) -> None:
+        assert self.data
+
         w = d.sympat + 3 * s
         s2 = (self.data[w + 2] << 4) | (self.data[w + 1] >> 4)
         if s2 == 0x0fff:
@@ -952,6 +958,8 @@ class Table:
         return idx
 
     def decompress_pairs(self, d: PairsData, idx: int) -> int:
+        assert self.data
+
         if not d.idxbits:
             return d.min_len
 
@@ -1017,16 +1025,16 @@ class Table:
             return self.data[w]
 
     def read_uint64_be(self, data_ptr: int) -> int:
-        return UINT64_BE.unpack_from(self.data, data_ptr)[0]
+        return UINT64_BE.unpack_from(self.data, data_ptr)[0]  # type: ignore
 
     def read_uint32(self, data_ptr: int) -> int:
-        return UINT32.unpack_from(self.data, data_ptr)[0]
+        return UINT32.unpack_from(self.data, data_ptr)[0]  # type: ignore
 
     def read_uint32_be(self, data_ptr: int) -> int:
-        return UINT32_BE.unpack_from(self.data, data_ptr)[0]
+        return UINT32_BE.unpack_from(self.data, data_ptr)[0]  # type: ignore
 
     def read_uint16(self, data_ptr: int) -> int:
-        return UINT16.unpack_from(self.data, data_ptr)[0]
+        return UINT16.unpack_from(self.data, data_ptr)[0]  # type: ignore
 
     def close(self) -> None:
         with self.write_lock:
@@ -1056,6 +1064,7 @@ class WdlTable(Table):
     def init_table_wdl(self) -> None:
         with self.write_lock:
             self.init_mmap()
+            assert self.data
 
             if self.initialized:
                 return
@@ -1078,8 +1087,6 @@ class WdlTable(Table):
             # Used if there are pawns.
             self.files = [PawnFileData() for _ in range(4)]
 
-            self.flags = None
-
             split = self.data[4] & 0x01
             files = 4 if self.data[4] & 0x02 else 1
 
@@ -1157,6 +1164,8 @@ class WdlTable(Table):
             self.initialized = True
 
     def setup_pieces_pawn(self, p_data: int, p_tb_size: int, f: int) -> None:
+        assert self.data
+
         j = 1 + int(self.pawns[1] > 0)
         order = self.data[p_data] & 0x0f
         order2 = self.data[p_data + 1] & 0x0f if self.pawns[1] else 0x0f
@@ -1175,6 +1184,8 @@ class WdlTable(Table):
         self.tb_size[p_tb_size + 1] = self.calc_factors_pawn(self.files[f].factor[1], order, order2, self.files[f].norm[1], f)
 
     def setup_pieces_piece(self, p_data: int) -> None:
+        assert self.data
+
         self.pieces[0] = [self.data[p_data + i + 1] & 0x0f for i in range(self.num)]
         order = self.data[p_data] & 0x0f
         self.set_norm_piece(self.norm[0], self.pieces[0])
@@ -1261,6 +1272,7 @@ class DtzTable(Table):
     def init_table_dtz(self) -> None:
         with self.write_lock:
             self.init_mmap()
+            assert self.data
 
             if self.initialized:
                 return
@@ -1278,24 +1290,24 @@ class DtzTable(Table):
             p_data = 5
 
             if not self.has_pawns:
-                self.map_idx = [0, 0, 0, 0]
+                self.map_idx = [[0, 0, 0, 0]]
 
                 self.setup_pieces_piece_dtz(p_data, 0)
                 p_data += self.num + 1
                 p_data += p_data & 0x01
 
                 self.precomp = self.setup_pairs(p_data, self.tb_size[0], 0, False)
-                self.flags = self._flags
+                self.flags: Union[int, List[int]] = self._flags
                 p_data = self._next
                 self.p_map = p_data
                 if self.flags & 2:
                     if not self.flags & 16:
                         for i in range(4):
-                            self.map_idx[i] = p_data + 1 - self.p_map
+                            self.map_idx[0][i] = p_data + 1 - self.p_map
                             p_data += 1 + self.data[p_data]
                     else:
                         for i in range(4):
-                            self.map_idx[i] = (p_data + 2 - self.p_map) // 2
+                            self.map_idx[0][i] = (p_data + 2 - self.p_map) // 2
                             p_data += 2 + 2 * self.read_uint16(p_data)
                 p_data += p_data & 0x01
 
@@ -1367,6 +1379,7 @@ class DtzTable(Table):
 
     def _probe_dtz_table(self, board: chess.Board, wdl: int) -> Tuple[int, int]:
         self.init_table_dtz()
+        assert self.data
 
         key = calc_key(board)
 
@@ -1384,6 +1397,8 @@ class DtzTable(Table):
             bside = 0
 
         if not self.has_pawns:
+            assert isinstance(self.flags, int)
+
             if (self.flags & 1) != bside and not self.symmetric:
                 return 0, -1
 
@@ -1404,13 +1419,15 @@ class DtzTable(Table):
 
             if self.flags & 2:
                 if not self.flags & 16:
-                    res = self.data[self.p_map + self.map_idx[WDL_TO_MAP[wdl + 2]] + res]
+                    res = self.data[self.p_map + self.map_idx[0][WDL_TO_MAP[wdl + 2]] + res]
                 else:
-                    res = self.read_uint16(self.p_map + 2 * (self.map_idx[WDL_TO_MAP[wdl + 2]] + res))
+                    res = self.read_uint16(self.p_map + 2 * (self.map_idx[0][WDL_TO_MAP[wdl + 2]] + res))
 
             if (not (self.flags & PA_FLAGS[wdl + 2])) or (wdl & 1):
                 res *= 2
         else:
+            assert isinstance(self.flags, list)
+
             k = self.files[0].pieces[0] ^ cmirror
             piece_type = k & 0x07
             color = k >> 3
@@ -1450,12 +1467,16 @@ class DtzTable(Table):
         return res, 1
 
     def setup_pieces_piece_dtz(self, p_data: int, p_tb_size: int) -> None:
+        assert self.data
+
         self.pieces = [self.data[p_data + i + 1] & 0x0f for i in range(self.num)]
         order = self.data[p_data] & 0x0f
         self.set_norm_piece(self.norm, self.pieces)
         self.tb_size[p_tb_size] = self.calc_factors_piece(self.factor, order, self.norm)
 
     def setup_pieces_pawn_dtz(self, p_data: int, p_tb_size: int, f: int) -> None:
+        assert self.data
+
         j = 1 + int(self.pawns[1] > 0)
         order = self.data[p_data] & 0x0f
         order2 = self.data[p_data + 1] & 0x0f if self.pawns[1] else 0x0f
@@ -1483,8 +1504,8 @@ class Tablebase:
         self.lru: Deque[Table] = collections.deque()
         self.lru_lock = threading.Lock()
 
-        self.wdl: MutableMapping[str, WdlTable] = {}
-        self.dtz: MutableMapping[str, DtzTable] = {}
+        self.wdl: Dict[str, Table] = {}
+        self.dtz: Dict[str, Table] = {}
 
     def _bump_lru(self, table: Table) -> None:
         if self.max_fds is None:
@@ -1500,7 +1521,7 @@ class Tablebase:
                 if len(self.lru) > self.max_fds:
                     self.lru.pop().close()
 
-    def _open_table(self, hashtable: MutableMapping[str, Table], Table: Type[Table], path: str) -> int:
+    def _open_table(self, hashtable: Dict[str, Table], Table: Type[Table], path: str) -> int:
         table = Table(path, variant=self.variant)
 
         if table.key in hashtable:
@@ -1560,7 +1581,7 @@ class Tablebase:
 
         key = calc_key(board)
         try:
-            table = self.wdl[key]
+            table = typing.cast(WdlTable, self.wdl[key])
         except KeyError:
             raise MissingTableError(f"did not find wdl table {key}")
 
@@ -1688,8 +1709,8 @@ class Tablebase:
             raise KeyError(f"syzygy tables do not contain positions with castling rights: {board.fen()}")
 
         # Validate piece count.
-        if chess.popcount(board.occupied) > 7:
-            raise KeyError(f"syzygy tables support up to 6 (and experimentally 7) pieces, not {chess.popcount(board.occupied)}: {board.fen()}")
+        if chess.popcount(board.occupied) > TBPIECES:
+            raise KeyError(f"syzygy tables support up to {TBPIECES} pieces, not {chess.popcount(board.occupied)}: {board.fen()}")
 
         # Probe.
         v, _ = self.probe_ab(board, -2, 2)
@@ -1733,7 +1754,7 @@ class Tablebase:
     def probe_dtz_table(self, board: chess.Board, wdl: int) -> Tuple[int, int]:
         key = calc_key(board)
         try:
-            table = self.dtz[key]
+            table = typing.cast(DtzTable, self.dtz[key])
         except KeyError:
             raise MissingTableError(f"did not find dtz table {key}")
 
@@ -1939,7 +1960,7 @@ class Tablebase:
 
         self.lru.clear()
 
-    def __enter__(self) -> "Tablebase":
+    def __enter__(self) -> Tablebase:
         return self
 
     def __exit__(self, exc_type: Optional[Type[BaseException]], exc_value: Optional[BaseException], traceback: Optional[TracebackType]) -> None:
@@ -1954,10 +1975,11 @@ def open_tablebase(directory: str, *, load_wdl: bool = True, load_dtz: bool = Tr
     .. note::
 
         Generally probing requires tablebase files for the specific
-        material composition, **as well as** tablebase files with less pieces.
-        This is important because 6-piece and 5-piece files are often
-        distributed seperately, but are both required for 6-piece positions.
-        Use :func:`~chess.syzygy.Tablebase.add_directory()` to load
+        material composition, **as well as** material compositions transitively
+        reachable by captures and promotions.
+        This is important because 6-piece and 5-piece (let alone 7-piece) files
+        are often distributed separately, but are both required for 6-piece
+        positions. Use :func:`~chess.syzygy.Tablebase.add_directory()` to load
         tables from additional directories.
     """
     tables = Tablebase(max_fds=max_fds, VariantBoard=VariantBoard)
